@@ -1,9 +1,8 @@
-from __future__ import absolute_import
-
 import logging
+from datetime import datetime, timezone
 from fnmatch import fnmatch
 
-from sentry_sdk.hub import Hub
+import sentry_sdk
 from sentry_sdk.utils import (
     to_string,
     event_from_exception,
@@ -11,11 +10,11 @@ from sentry_sdk.utils import (
     capture_internal_exceptions,
 )
 from sentry_sdk.integrations import Integration
-from sentry_sdk._compat import iteritems, utc_from_timestamp
 
-from sentry_sdk._types import TYPE_CHECKING
+from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from collections.abc import MutableMapping
     from logging import LogRecord
     from typing import Any
     from typing import Dict
@@ -91,6 +90,10 @@ class LoggingIntegration(Integration):
 
         def sentry_patched_callhandlers(self, record):
             # type: (Any, LogRecord) -> Any
+            # keeping a local reference because the
+            # global might be discarded on shutdown
+            ignored_loggers = _IGNORED_LOGGERS
+
             try:
                 return old_callhandlers(self, record)
             finally:
@@ -98,15 +101,17 @@ class LoggingIntegration(Integration):
                 # the integration.  Otherwise we have a high chance of getting
                 # into a recursion error when the integration is resolved
                 # (this also is slower).
-                if record.name not in _IGNORED_LOGGERS:
-                    integration = Hub.current.get_integration(LoggingIntegration)
+                if ignored_loggers is not None and record.name not in ignored_loggers:
+                    integration = sentry_sdk.get_client().get_integration(
+                        LoggingIntegration
+                    )
                     if integration is not None:
                         integration._handle_record(record)
 
         logging.Logger.callHandlers = sentry_patched_callhandlers  # type: ignore
 
 
-class _BaseHandler(logging.Handler, object):
+class _BaseHandler(logging.Handler):
     COMMON_RECORD_ATTRS = frozenset(
         (
             "args",
@@ -152,10 +157,10 @@ class _BaseHandler(logging.Handler, object):
         )
 
     def _extra_from_record(self, record):
-        # type: (LogRecord) -> Dict[str, None]
+        # type: (LogRecord) -> MutableMapping[str, object]
         return {
             k: v
-            for k, v in iteritems(vars(record))
+            for k, v in vars(record).items()
             if k not in self.COMMON_RECORD_ATTRS
             and (not isinstance(k, str) or not k.startswith("_"))
         }
@@ -179,11 +184,11 @@ class EventHandler(_BaseHandler):
         if not self._can_record(record):
             return
 
-        hub = Hub.current
-        if hub.client is None:
+        client = sentry_sdk.get_client()
+        if not client.is_active():
             return
 
-        client_options = hub.client.options
+        client_options = client.options
 
         # exc_info might be None or (None, None, None)
         #
@@ -197,7 +202,7 @@ class EventHandler(_BaseHandler):
                 client_options=client_options,
                 mechanism={"type": "logging", "handled": True},
             )
-        elif record.exc_info and record.exc_info[0] is None:
+        elif (record.exc_info and record.exc_info[0] is None) or record.stack_info:
             event = {}
             hint = {}
             with capture_internal_exceptions():
@@ -221,7 +226,9 @@ class EventHandler(_BaseHandler):
 
         hint["log_record"] = record
 
-        event["level"] = self._logging_to_event_level(record)
+        level = self._logging_to_event_level(record)
+        if level in {"debug", "info", "warning", "error", "critical", "fatal"}:
+            event["level"] = level  # type: ignore[typeddict-item]
         event["logger"] = record.name
 
         # Log records from `warnings` module as separate issues
@@ -241,12 +248,16 @@ class EventHandler(_BaseHandler):
         else:
             event["logentry"] = {
                 "message": to_string(record.msg),
-                "params": record.args,
+                "params": (
+                    tuple(str(arg) if arg is None else arg for arg in record.args)
+                    if record.args
+                    else ()
+                ),
             }
 
         event["extra"] = self._extra_from_record(record)
 
-        hub.capture_event(event, hint=hint)
+        sentry_sdk.capture_event(event, hint=hint)
 
 
 # Legacy name
@@ -271,7 +282,7 @@ class BreadcrumbHandler(_BaseHandler):
         if not self._can_record(record):
             return
 
-        Hub.current.add_breadcrumb(
+        sentry_sdk.add_breadcrumb(
             self._breadcrumb_from_record(record), hint={"log_record": record}
         )
 
@@ -282,6 +293,6 @@ class BreadcrumbHandler(_BaseHandler):
             "level": self._logging_to_event_level(record),
             "category": record.name,
             "message": record.message,
-            "timestamp": utc_from_timestamp(record.created),
+            "timestamp": datetime.fromtimestamp(record.created, timezone.utc),
             "data": self._extra_from_record(record),
         }
